from loguru import logger


# context
#   boxes: structure of boxes
#   client: MQTT client
#
# boxes['box_name']['context']
#   store here what ever is require to represent the state of the box

def prepare_context(box_name, context):
    local_context = {}
    
    local_context['id'] = box_name
    local_context['label'] = context['boxes'][box_name]['label']
    local_context['high_temperature'] = context['default_high_temperature']
    local_context['low_temperature'] = context['low_temperature']
    local_context['off_temperature'] = context['off_temperature']
    local_context['maintenance_temperature'] = context['maintenance_temperature']

    local_context['general_off'] = False
    local_context['maintenance_mode'] = False

    local_context['window_state'] = {}
    for w in context['boxes'][box_name]['windows']:
        local_context['window_state'][w['label']] = 'closed'

    local_context['output_temperature'] = local_context['high_temperature']

    return local_context

def process_message(box_name, topic_key, payload, context):
    try:
        box = context['boxes'][box_name]
        local_context = box['context']
        logger.info(f"{local_context=}")
        logger.info(f"[{box_name}, {box['label']}] Processing message for '{topic_key}': {payload}")

        match topic_key.split('/'):
            case [ primary_key, sub_key ] if primary_key == 'window':
                result = process_window(box_name, context, local_context, sub_key, payload)
            case [ primary_key ] if primary_key == 'high_temp':
                result = process_high_temp(box_name, context, local_context, payload)
            case [ primary_key ] if primary_key == 'cmd':
                result = process_cmd(box_name, context, local_context, payload)
            case [ primary_key ] if primary_key == 'general_off':
                result = process_general_off(box_name, context, local_context, payload)
            case [ primary_key ] if primary_key == 'maintenance_mode':
                result = process_maintenance_mode(box_name, context, local_context, payload)
            case [ primary_key ] if primary_key == 'status':
                result = process_status(box_name, context, local_context, payload)
            case _:
                raise Error(f"Unexcepted topic_key: {topic_key}, {payload}") 

        if result:
            (result_message, status) = result
            publish_topic = box["output_topic"] if not status else context['status_topic']
            context['client'].publish(publish_topic, result_message)
            logger.info(f"[{box_name}] Result published on '{publish_topic}': {status} {result_message}")
    except Exception as e:
        logger.error(f"[{box_name}] Error processing '{topic_key}': {e}") 


def process_general_off(box_name, context, local_context, payload):
    return ("general off", False)

def process_maintenance_mode(box_name, context, local_context, payload):
    return ("maintenance mode", False)

def process_status(box_name, context, local_context, payload):
    return (f"{local_context}", True)

def process_cmd(box_name, context, local_context, payload):
    return (f"cmd: {payload}", False)

def process_high_temp(box_name, context, local_context, payload):
    return (f"high_temp: {payload}", False)

def process_window(box_name, context, local_context, sub_key, payload):
    return (f"window: {sub_key}, {payload}", False)
